clc; clear;

% Lognormal mixture model of Brigo, Mercurio, and Rapidsada on SP500.
% by Fabrice Douglas Rouah, FRouah.com and Volopta.com

BSCall = @(s,K,r,q,v,T) s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T));


%% Call option data
IV = [ ...
   27.8000   26.3800   25.3200   25.1800
   24.7700   24.0200   23.6400   23.6900
   21.8600   21.5800   22.0300   22.3900
   18.7800   19.3000   20.4700   20.9800
   15.7200   17.1200   18.9400   19.7000
   13.3400   15.1700   17.4800   18.4900
   13.2300   13.7300   16.1800   17.3600]./100;
Days = [45 98 261 348];
T = Days./365;
K = [120   125   130   135   140   145   150];
[NK NT] = size(IV);
r = 0.0010;
q  = 0.0068;
S0 = 137.14;

% Market call prices
PutCall = 'C';
for t=1:NT
    for k=1:NK
        MktPrice(k,t) = BSCall(S0,K(k),r,q,IV(k,t),T(t));
    end;
end;

%% Find parameter estimates
% Number of lognormal mixtures
N = 3;

% Starting values of parameters
start = [1/N.*ones(1,N-1) 2.*ones(1,5*N)];

% Optimization step
[param feval] = fminsearch(@(p) findBMParams(p,PutCall,S0,K,r,q,T,N,MktPrice),start);

% Separate out the parameters
% Lambda weights
L = param(1:N-1);
complement = sum(L);
L(N) = 1 - complement;

% Shift parameters
beta = param(N:2*N-1);

% Parameters for eta function
a   = param(2*N:3*N-1);
b   = param(3*N:4*N-1);
c   = param(4*N:5*N-1);
tau = param(5*N:6*N-1);

%% Print out the parameter estimates
clc;
fprintf('Brigo, Mercurio, Rapisarda parameter estimation \n')
fprintf('----------------------------------------------------------------\n')
fprintf('Num    Beta       a          b        c        tau      lambda \n')
fprintf('----------------------------------------------------------------\n')
for k=1:N
    fprintf('%3.0f %8.4f  %8.4f  %8.4f  %8.4f  %8.4f  %8.4f \n',...
        k,beta(k),a(k),b(k),c(k),tau(k),L(k));
end
fprintf('----------------------------------------------------------------\n')
fprintf('Sum of weights %8.4f \n',sum(L));

%% Obtain the fitted model prices
NK = length(K);
NT = length(T);
for k=1:NK;
    for t=1:NT;
        ModelPrice(k,t) = max(0.05,BMPrice(L,beta,a,b,c,tau,PutCall,S0,K(k),r,q,T(t),N));
    end
end

%% Print out the prices
fprintf('\n')
fprintf('Market Prices\n')
fprintf(' Strike    45days      98days      261days     348days \n');
fprintf('-------------------------------------------------------\n')
for k=1:NK
    fprintf(' %5.0f  %10.4f  %10.4f  %10.4f  %10.4f \n',K(k),MktPrice(k,:))
end
fprintf('-------------------------------------------------------\n')
fprintf('Model Prices\n')
fprintf(' Strike    45days      98days      261days     348days \n');
fprintf('-------------------------------------------------------\n')
for k=1:NK
    fprintf(' %5.0f  %10.4f  %10.4f  %10.4f  %10.4f \n',K(k),ModelPrice(k,:))
end
fprintf('-------------------------------------------------------\n')

%% Extract and plot the Risk neutral densities
% Range of stock prices
S = 75:225;

% Calculate the RND
for t=1:NT
    for k=1:length(S)
        y = S(k);
        RND(k,t) = BMpdf2(y,S0,r,q,T(t),L,beta,a,b,c,tau);
    end
end

subplot(2,1,1)
plot(S,RND)
legend('45 days','98 days','261 days','348 days');    
title('Lognormal mixture risk neutral density')

%% Extract and plot the eta integrated volatilities
NTi = 100;
dT = (T(end)-T(1))/(NTi-1);
Ti = T(1):dT:T(end);
 
for t=1:length(Ti);
    for i=1:N
        eta(t,i) = a(i) + b(i)*(1-exp(-Ti(t)/tau(i)))*(tau(i)/Ti(t)) + c(i)*exp(-Ti(t)/tau(i));
    end
end

subplot(2,1,2)
plot(Ti,eta);
title('Integrated volatility functions')
